@@ -1,5 +1,5 @@
 /*
- Copyright (©) 2003-2018 Teus Benschop.
+ Copyright (©) 2003-2020 Teus Benschop.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -98,7 +98,7 @@ void client_index_enable_client (void * webserver_request, string username, stri
   }
   
   // Schedule a sync operation straightaway.
-  sendreceive_queue_sync (-1);
+  sendreceive_queue_sync (-1, 0);
 }
 
 
@@ -176,6 +176,10 @@ string client_index (void * webserver_request)
 
   view.set_variable ("external", assets_external_logic_link_addon ());
 
+  if (request->query.count ("info")) {
+    view.enable_zone ("info");
+  }
+  
   string page;
 
   // Since the role of the user may change after a successful connection to the server,
