@@ -1,5 +1,5 @@
 /*
-Copyright (©) 2003-2019 Teus Benschop.
+Copyright (©) 2003-2020 Teus Benschop.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #include <database/config/user.h>
 #include <cstdlib>
 #include <webserver/request.h>
+#include <config/globals.h>
 
 
 const char * config_logic_config_folder ()
@@ -40,6 +41,16 @@ const char * config_logic_version ()
 }
 
 
+// Loads the values from the config folder into memory for faster access.
+void config_logic_load_settings ()
+{
+  string path;
+  // Read the setting whether to log network connections.
+  path = filter_url_create_root_path (config_logic_config_folder (), "log-net");
+  config_globals_log_incoming_connections = file_or_dir_exists (path);
+}
+
+
 // Return the network port configured for the server.
 string config_logic_http_network_port ()
 {
@@ -250,3 +261,10 @@ void config_logic_swipe_enabled (void * webserver_request, string & script)
   script.append (true_false);
   script.append (";");
 }
+
+
+// Whether to log incoming IP connections to the journal.
+bool config_logic_log_incoming_connections ()
+{
+  return config_globals_log_incoming_connections;
+}
