@@ -1,5 +1,5 @@
 /*
-Copyright (©) 2003-2019 Teus Benschop.
+Copyright (©) 2003-2020 Teus Benschop.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 var quill = undefined;
 var Delta = Quill.import ("delta");
+var chapterEditorUniqueID = Math.floor (Math.random() * 100000000);
 
 
 $ (document).ready (function ()
@@ -54,6 +55,9 @@ $ (document).ready (function ()
   }
 
   $ ("#editor").on ("click", editorNoteCitationClicked);
+
+  $ ("#editor").bind ("paste", editorClipboardPasteHandler);
+  
 });
 
 
@@ -90,7 +94,7 @@ function editorInitialize ()
     return new Delta().insert (plaintext);
   });
 
-  if (!quill.hasFocus ()) quill.focus ();
+  if (editorWriteAccess) if (!quill.hasFocus ()) quill.focus ();
   
   // Event handlers.
   quill.on ("text-change", editorTextChangeHandler);
@@ -98,6 +102,18 @@ function editorInitialize ()
 }
 
 
+// This fixes an issue where upon pasting text in the editor,
+// the editor scrolls to the very top of the text.
+// If the text is long, then the focused verse is thrown off the screen.
+// Here it fixes that.
+// https://github.com/bibledit/cloud/issues/428
+function editorClipboardPasteHandler (event)
+{
+  var currentScrollTop = $("#workspacewrapper").scrollTop();
+  $("#workspacewrapper").animate({ scrollTop: currentScrollTop }, 100);
+}
+
+
 /*
 
 Section for the new Passage event from the Navigator.
@@ -123,9 +139,13 @@ function navigationNewPassage ()
   } else {
     return;
   }
-
+  
   if ((editorNavigationBook != editorLoadedBook) || (editorNavigationChapter != editorLoadedChapter)) {
+    // Fixed: Reload text message when switching to another chapter.
+    // https://github.com/bibledit/cloud/issues/408
+    editorSaveDate = new Date(0);
     editorSaveChapter ();
+    editorSaveDate = new Date(0);
     editorLoadChapter (false);
   } else {
     editorScheduleCaretPositioning ();
@@ -174,15 +194,28 @@ function editorLoadChapter (reload)
   $.ajax ({
     url: "load",
     type: "GET",
-    data: { bible: editorLoadedBible, book: editorLoadedBook, chapter: editorLoadedChapter },
+    data: { bible: editorLoadedBible, book: editorLoadedBook, chapter: editorLoadedChapter, id: chapterEditorUniqueID },
     success: function (response) {
       // Set the editor read-write or read-only.
       editorWriteAccess = checksum_readwrite (response);
+      // If this is the second or third or higher editor in the workspace,
+      // make the editor read-only.
+      if (window.frameElement) {
+        iframe = $(window.frameElement);
+        var data_editor_number = iframe.attr("data-editor-no");
+        if (data_editor_number > 1) {
+          editorWriteAccess = false;
+        }
+      }
       // Checksumming.
       response = checksum_receive (response);
       if (response !== false) {
         // Only load new text when it is different.
-        if (response != editorGetHtml ()) {
+        // Extract the plain text from the html and compare that.
+        // https://github.com/bibledit/cloud/issues/449
+        var responseText = $(response).text();
+        var editorText = $(editorGetHtml ()).text();
+        if (responseText != editorText) {
           // Destroy existing editor.
           if (quill) delete quill;
           // Load the html in the DOM.
@@ -195,18 +228,21 @@ function editorLoadChapter (reload)
           css4embeddedstyles ();
           // Feedback.
           editorStatus (editorChapterLoaded);
-        }
-        // Reference for comparison at save time.
-        editorReferenceText = editorGetHtml ();
-        // Position caret straightaway.
-        if (reload) {
-          positionCaret (editorCaretPosition);
-        }
-        editorScheduleCaretPositioning ();
-        editorLoadDate = new Date();
-        var seconds = (editorLoadDate.getTime() - editorSaveDate.getTime()) / 1000;
-        if (seconds < 2) {
-          alert (editorChapterVerseUpdatedLoaded);
+          // Reference for comparison at save time.
+          editorReferenceText = editorGetHtml ();
+          // Position caret straightaway.
+          if (reload) {
+            positionCaret (editorCaretPosition);
+          }
+          editorScheduleCaretPositioning ();
+          // Alert on reloading soon after save, or after text reload.
+          // https://github.com/bibledit/cloud/issues/346
+          editorLoadDate = new Date();
+          var seconds = (editorLoadDate.getTime() - editorSaveDate.getTime()) / 1000;
+          seconds = 2; // Disable timer.
+          if ((seconds < 2) || reload) {
+            if (editorWriteAccess) editorReloadAlert (editorChapterVerseUpdatedLoaded);
+          }
         }
       } else {
         // Checksum error: Reload.
@@ -243,7 +279,7 @@ function editorSaveChapter (sync)
     url: "save",
     type: "POST",
     async: editorSaveAsync,
-    data: { bible: editorLoadedBible, book: editorLoadedBook, chapter: editorLoadedChapter, html: encodedHtml, checksum: checksum },
+    data: { bible: editorLoadedBible, book: editorLoadedBook, chapter: editorLoadedChapter, html: encodedHtml, checksum: checksum, id: chapterEditorUniqueID },
     success: function (response) {
       editorStatus (response);
       if (response == editorChapterReformat) {
@@ -262,8 +298,9 @@ function editorSaveChapter (sync)
       editorSaving = false;
       editorSaveDate = new Date();
       var seconds = (editorSaveDate.getTime() - editorLoadDate.getTime()) / 1000;
+      seconds = 2; // Disable timer.
       if (seconds < 2) {
-        alert (editorChapterVerseUpdatedLoaded);
+        if (editorWriteAccess) editorReloadAlert (editorChapterVerseUpdatedLoaded);
       }
     },
   });
@@ -291,7 +328,7 @@ function editorGetHtml ()
 
 /*
 
-Portion dealing with triggers for editor's content change.
+Portion dealing with triggers for when the editor's content changes.
 
 */
 
@@ -337,6 +374,18 @@ Section for polling the server for updates on the loaded chapter.
 
 var editorChapterIdOnServer = 0;
 var editorChapterIdPollerTimeoutId;
+var editorChapterIdAjaxRequest;
+
+
+function editorIdPollerTimeoutStop ()
+{
+  if (editorChapterIdPollerTimeoutId) {
+    clearTimeout (editorChapterIdPollerTimeoutId);
+  }
+  if (editorChapterIdAjaxRequest && editorChapterIdAjaxRequest.readystate != 4) {
+    editorChapterIdAjaxRequest.abort();
+  }
+}
 
 
 function editorIdPollerTimeoutStart ()
@@ -346,15 +395,16 @@ function editorIdPollerTimeoutStart ()
 }
 
 
-function editorIdPollerTimeoutStop ()
-{
-  if (editorChapterIdPollerTimeoutId) clearTimeout (editorChapterIdPollerTimeoutId);
-}
-
-
 function editorEditorPollId ()
 {
-  $.ajax ({
+  // Due to network latency, there may be multiple ongoing polls.
+  // Multiple polls may return multiple chapter identifiers.
+  // This could lead to false "text reloaded" notifications.
+  // https://github.com/bibledit/cloud/issues/424
+  // To handle this, switch the poller off.
+  editorIdPollerTimeoutStop ();
+
+  editorChapterIdAjaxRequest = $.ajax ({
     url: "id",
     type: "GET",
     data: { bible: editorLoadedBible, book: editorLoadedBook, chapter: editorLoadedChapter },
@@ -370,7 +420,9 @@ function editorEditorPollId ()
       }
     },
     complete: function (xhr, status) {
-      editorIdPollerTimeoutStart ();
+      if (status != "abort") {
+        editorIdPollerTimeoutStart ();
+      }
     }
   });
 }
@@ -657,7 +709,7 @@ function editorScrollVerseIntoView ()
   var bounds = quill.getBounds (position);
   var workspaceHeight = $("#workspacewrapper").height();
   var currentScrollTop = $("#workspacewrapper").scrollTop();
-  var scrollTo = bounds.top - (workspaceHeight /  2);
+  var scrollTo = bounds.top - (workspaceHeight * verticalCaretPosition / 100);
   scrollTo = parseInt (scrollTo);
   var lowerBoundary = currentScrollTop - (workspaceHeight / 10);
   var upperBoundary = currentScrollTop + (workspaceHeight / 10);
@@ -900,14 +952,9 @@ function applyNotesStyle (style)
   quill.setSelection (caret, 0);
   quill.insertText (caret, caller, "character", "notecall" + noteId, "user");
 
-  // Append note text to notes section:
-  // <p class="b-f"><span class="i-notebody1">1</span> + .</p>
-  var length = quill.getLength ();
-  quill.insertText (length, "\n", "paragraph", style, "user");
-  quill.insertText (length, caller, "character", "notebody" + noteId, "user");
-  length++;
-  quill.insertText (length, " + .", "character", "", "user");
-  
+  // Append note text to notes section.
+  assetsEditorAddNote (quill, style, caller, noteId, editorNavigationChapter, verseSeparator, editorNavigationVerse);
+
   editorInsertedNotesCount++;
   
   editorActiveStylesFeedback ();
@@ -1043,3 +1090,28 @@ function editorSwipeRight (event)
 }
 
 
+/*
+
+Section for reload notifications.
+
+*/
+
+
+function editorReloadAlert (message)
+{
+  return;
+  // Take action only if the editor has focus and the user can type in it.
+  if (quill.hasFocus ()) {
+    notifyItSuccess (message)
+    quill.enable (false);
+    setTimeout (editorReloadAlertTimeout, 3000);
+  }
+}
+
+
+function editorReloadAlertTimeout ()
+{
+  quill.enable (editorWriteAccess);
+  quill.focus ();
+}
+
