@@ -1,5 +1,5 @@
 /*
- Copyright (©) 2003-2019 Teus Benschop.
+ Copyright (©) 2003-2020 Teus Benschop.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -34,7 +34,6 @@ class Editor_Html2Usfm
 public:
   void load (string html);
   void stylesheet (string stylesheet);
-  void quill ();
   void run ();
   string get ();
 private:
@@ -47,6 +46,7 @@ private:
   set <string> noteOpeners;
   vector <string> characterStyles; // Active character styles.
   bool processingNote = false; // Note processing flag.
+  string lastNoteStyle; // The most recent style opened inside a note.
   void preprocess ();
   void flushLine ();
   void postprocess ();
@@ -57,8 +57,7 @@ private:
   void openInline (string className);
   void processNoteCitation (xml_node node);
   string cleanUSFM (string usfm);
-  xml_node get_note_pointer (xml_node node, string id);
-  bool quill_enabled = false;
+  xml_node get_note_pointer (xml_node body, string id);
   string update_quill_class (string classname);
 };
 
