@@ -35,7 +35,7 @@ $(document).ready (function () {
     e.preventDefault();
     document.execCommand ("insertHTML", false, data);
   });
-  usfmIdPoller ();
+  usfmIdPollerOn ();
   $ ("#usfmeditor").on ("paste cut click", usfmCaretChanged);
   $ ("#usfmeditor").on ("keydown", usfmHandleKeyDown);
   if (usfmEditorWriteAccess) $ ("#usfmeditor").focus ();
@@ -62,7 +62,6 @@ var usfmNavigationVerse;
 var usfmEditorChangedTimeout;
 var usfmLoadedText;
 var usfmIdChapter = 0;
-var usfmIdTimeout;
 var usfmCaretTimeout;
 var usfmReload = false;
 var usfmCaretPosition = 0;
@@ -90,6 +89,11 @@ function navigationNewPassage ()
   } else {
     return;
   }
+  if ((usfmNavigationBook != usfmBook) || (usfmNavigationChapter != usfmChapter)) {
+    // Fixed: Reload text message when switching to another chapter.
+    // https://github.com/bibledit/cloud/issues/408
+    usfmSaveDate = new Date(0);
+  }
   usfmEditorSaveChapter ();
   usfmReload = false;
   usfmEditorLoadChapter ();
@@ -145,8 +149,9 @@ function usfmEditorLoadChapter ()
           // https://github.com/bibledit/cloud/issues/346
           usfmLoadDate = new Date();
           var seconds = (usfmLoadDate.getTime() - usfmSaveDate.getTime()) / 1000;
+          seconds = 2; // Disable timer.
           if ((seconds < 2) | usfmReload) {
-            if (usfmEditorWriteAccess) usfmEditorReloadAlert (usfmEditorVerseUpdatedLoaded); // Todo
+            if (usfmEditorWriteAccess) usfmEditorReloadAlert (usfmEditorVerseUpdatedLoaded);
           }
           usfmReload = false;
         } else {
@@ -169,6 +174,9 @@ function usfmEditorUnload ()
 
 function usfmEditorSaveChapter (sync)
 {
+  // Fix for https://github.com/bibledit/cloud/issues/427
+  usfmEditorStatus ("");
+
   if (usfmSaving) {
     usfmEditorChanged ();
     return;
@@ -207,8 +215,9 @@ function usfmEditorSaveChapter (sync)
       usfmSaving = false;
       usfmSaveDate = new Date();
       var seconds = (usfmSaveDate.getTime() - usfmLoadDate.getTime()) / 1000;
+      seconds = 2; // Disable timer.
       if (seconds < 2) {
-        if (usfmEditorWriteAccess) usfmEditorReloadAlert (usfmEditorVerseUpdatedLoaded); // Todo
+        if (usfmEditorWriteAccess) usfmEditorReloadAlert (usfmEditorVerseUpdatedLoaded);
       }
     }
   });
@@ -242,21 +251,36 @@ function usfmEditorSelectiveNotification (message)
   if (message == usfmEditorWillSave) return;
   if (message == usfmEditorChapterSaving) return;
   if (message == usfmEditorChapterSaved) return;
+  if (message == "") return;
   notifyItError (message);
 }
 
 
-function usfmIdPoller ()
+var usfmIdTimeout;
+var usfmIdAjaxRequest;
+
+
+function usfmIdPollerOff ()
 {
   if (usfmIdTimeout) {
     clearTimeout (usfmIdTimeout);
   }
+  if (usfmIdAjaxRequest && usfmIdAjaxRequest.readystate != 4) {
+    usfmIdAjaxRequest.abort();
+  }
+}
+
+
+function usfmIdPollerOn ()
+{
+  usfmIdPollerOff ();
   usfmIdTimeout = setTimeout (usfmEditorPollId, 1000);
 }
 
 
 function usfmEditorPollId ()
 {
+  usfmIdPollerOff ();
   $.ajax ({
     url: "../edit/id",
     type: "GET",
@@ -275,7 +299,9 @@ function usfmEditorPollId ()
       }
     },
     complete: function (xhr, status) {
-      usfmIdPoller ();
+      if (status != "abort") {
+        usfmIdPollerOn ();
+      }
     }
   });
 }
@@ -533,7 +559,7 @@ Section for reload notifications.
 */
 
 
-function usfmEditorReloadAlert (message) // Todo
+function usfmEditorReloadAlert (message)
 {
   // Take action only if the editor has focus and the user can type in it.
   var editor = $ ("#usfmeditor");
