@@ -1,5 +1,5 @@
 /*
- Copyright (©) 2003-2019 Teus Benschop.
+ Copyright (©) 2003-2020 Teus Benschop.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -48,9 +48,10 @@ string editusfm_load (void * webserver_request)
   string bible = request->query ["bible"];
   int book = convert_to_int (request->query ["book"]);
   int chapter = convert_to_int (request->query ["chapter"]);
-  
+  string unique_id = request->query ["id"];
+
   // Store a copy of the USFM loaded in the editor for later reference.
-  storeLoadedUsfm (webserver_request, bible, book, chapter, "editusfm");
+  storeLoadedUsfm (webserver_request, bible, book, chapter, unique_id);
 
   string usfm = request->database_bibles()->getChapter (bible, book, chapter);
 
